
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		const_def.h
*	Project:	MCRP09_RX200 (ss) (YRotateItRX200_1s_V1)
*	Version:	1.0
*	Date:		19-11-2013 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	Internal constants definition
*******************************************************************************/

#ifndef _CONST_DEF_H
#define _CONST_DEF_H

#include "typedefine.h"
#include "customize.h"

// base timings
#define MTU_FRE_MHZ		( 32.0 )	// MTU frequency in MHz
#define	SAM_FRE_KHZ		( 8.0 )		// desired sampling frequency in kHz (4.0<=SAM_FRE_KHZ<=8.0)
#define FRE_RATIO		( 2 )		// PWM_FRE_KHZ=FRE_RATIO*SAM_FRE_KHZ (1<=FRE_RATIO<=8)
#define	MLOOP_FRE_KHZ	( 0.1 )		// desired main loop frequency in kHz

// derived timings
#if (FRE_RATIO == 1)
#define	INT_SKIP		( 0 )
#else	// if (FRE_RATIO == 1)
#define	INT_SKIP		((uint32_t)(7 + FRE_RATIO))				// (0x10=8) + (TC_RATIO-1)
#endif	// if (FRE_RATIO == 1)
#define MTU_FRE_KHZ		(1000.0 * MTU_FRE_MHZ)
#define MTU_FRE_HZ		(1000.0 * MTU_FRE_KHZ)
#define PWM_FRE_KHZ		(SAM_FRE_KHZ * FRE_RATIO)
#define PWM_SEMIPER_INT	((uint32_t)(0.5 * MTU_FRE_KHZ / PWM_FRE_KHZ))// half PWM period expressed as number of MTU ticks
#define PWM_PER_INT		((uint32_t)(2 * PWM_SEMIPER_INT))		// PWM period expressed as number of MTU ticks
#define SAM_PER_INT		((uint32_t)(FRE_RATIO * PWM_PER_INT))	// Sampling period expressed as number of MTU ticks
#define	PWM_FRE_HZ		(MTU_FRE_HZ / PWM_PER_INT)				// actual pwm frequency in Hz
#define	PWM_PER_SEC		(PWM_PER_INT / MTU_FRE_HZ)				// actual pwm period in sec
#define	SAM_FRE_HZ		(MTU_FRE_HZ / SAM_PER_INT)				// actual sampling frequency in Hz
#define	SAM_PER_SEC		(SAM_PER_INT / MTU_FRE_HZ)				// actual sampling period in sec
#define NUM_INT			((uint16_t)(SAM_FRE_HZ / (MLOOP_FRE_KHZ * 1000.0)))
#define	MLOOP_FRE_HZ	(SAM_FRE_HZ / NUM_INT)					// actual main loop frequency in Hz
#define	MLOOP_PER_SEC	(NUM_INT / MTU_FRE_HZ)					// actual main loop period in sec
#define	MLOOP_PER_MS	((uint16_t)(1000.0 / MLOOP_FRE_HZ))		// main loop period in msec
#define CNT_MS( X )		((uint32_t)(( X ) * SAM_FRE_HZ / 1000.0))	// n. of sampling periods corr. to Xms

// PWM parameters
#define	SEMIPER			PWM_SEMIPER_INT						// half PWM period expressed as number of MTU ticks
#if (DEADTIM_NS<=31)	// 31=(1000000000/MTU_FRE_HZ)
#define DEADTIM			( 1 )
#else	// if (DEADTIM_NS<=31)
#define DEADTIM			((uint16_t)(MTU_FRE_HZ * (DEADTIM_NS / 1000000000.0)))
#endif	// if (DEADTIM_NS<=31)
#define	SEMIPERDEAD		((uint16_t)(SEMIPER + DEADTIM))
#define PWM_CENTER		((uint16_t)(SEMIPERDEAD >> 1))

// minimum on time of low arms, to guarantee bootstrap operations (LARGER than DEADTIM!)
#define MIN_TON_LOW		((uint16_t)(MTU_FRE_HZ * (MIN_TON_LOW_NS / 1000000000.0)))

// single shunt timings
#define SAMP_SS_NS		( 1000 )
#define SAMP_SS			((u16)(MTU_FRE_HZ * (SAMP_SS_NS / 1000000000.0)))
#define DEL_SS			(DEADTIM + SAMP_SS)
#define STAB_SS_NS		( 3000 )
#define STAB_SS			((u16)(MTU_FRE_HZ * (STAB_SS_NS / 1000000000.0)))
#define MIN_SS			(DEADTIM + SAMP_SS + STAB_SS)

// maximum delta duty and default effective output voltage ratio
#define DELMAX			((uint16_t)(SEMIPER - MIN_TON_LOW))
#define KVOUT			((uint16_t)((NORMVAL * DELMAX) / SEMIPER))
#define K_VMAX			((u16)((ONEDIVSQRT3 * DELMAX) / SEMIPER))	// max out voltage (without hexagonal saturation) is VB/SQRT3
#define K_VMAX_OVM		((u16)((TWOTHIRDS * DELMAX) / SEMIPER))		// max out voltage (with hexagonal saturation) is 2*VB/3

// A/D converter voltage and range
#define ADVCC			( 5.0 )			// A/D Range V
#define ADFS			( 4096.0 )		// A/D full scale (bit)

// reference voltage
#define VREF_VOL		( 4.25 )		// reference voltage value
#define VREF_AD			((uint16_t)(VREF_VOL * ADFS / ADVCC))
#define SH_ADCOMP		( 14 )
#define AMP_ADCOMP		(1 << SH_ADCOMP)
#define KADCOMP			((uint32_t)(AMP_ADCOMP * VREF_AD))

// output currents A/D conversion (the reading is referred to AVCC/2, so half the range is used)
#define RAV_AD			(1.0 /(RSHUNT_OHM * RSGAIN))	// peak real_ampere/pin_volt ratio at the input pin

// voltages (bus and output) A/D conversion (the reading is referred to 0, so all the range is used)
#define RVV_AD			((RVBUSH_OHM + RVBUSL_OHM) / RVBUSL_OHM)	// peak real_volt/pin_volt ratio at the input pin

// two pi
#ifndef TWOPIF
#define TWOPIF			( 6.283185307 )
#endif	// TWOPIF

// base value for normalization
#ifndef NORMVAL
#define SH_NORMVAL		( 14 )			// NORMVAL=2^SHFT_NORMVAL
#define NORMVAL			((int16_t)( 1 << SH_NORMVAL ))
#define NORMVALF		((float)NORMVAL)
#endif	// NORMVAL

// numerical constants
#ifndef S16_CONST
#define S16_CONST
#define	SH_S16_CONST	( SH_NORMVAL )	// amplification shifts
#define	AMP_S16_CONST	( 1 << SH_S16_CONST )	// amplification
#define	ONETHIRD		( 5461 )		// 2^14*(1/3)=5461.33333
#define	TWOTHIRDS		( 10923 )		// 2^14*(2/3)=10922.66666
#define	SQRT3			( 28378 )		// 2^14*SQUAREROOT(3)=28377.92043
#define	TWODIVSQRT3		( 18919 )		// 2^15/SQUAREROOT(3)=18918.61362
#define	ONEDIVSQRT3		( 9459 )		// 2^14/SQUAREROOT(3)=9459.30681
#define	SQRT2			( 23170 )		// 2^14*SQUAREROOT(2)=23170.47501
#define	ONEDIVSQRT2		( 11585 )		// 2^14/SQUAREROOT(2)=11585.2375
#define SQRT3DIVSQRT2	( 20066 )		// 2^14*SQUAREROOT(3/2)=20066.21997
#define SQRT2DIVSQRT3	( 13377 )		// 2^14*SQUAREROOT(2/3)=13377.47998
#endif	// S16_CONST

/*
Maximum values definitions: regarding voltages and currents, the maximum values which can
be read from the A/D converter should be chosen, while regarding the frequency, the maximum
value which can be obtained from the motor given the voltage and the current has to be
chosen (nominal freq); in case of PM motors with heavy flux weakening, the maximum speed
requested by the application. In case of deep flux weakening, the base impedence has to
be chosen greater.
*/
#define	MAX_VOLTAGE_VOL			(RVV_AD * ADVCC)		// maximum allowable voltage (full A/D range)
#define	MAX_CURRENT_AMP			(RAV_AD * (ADVCC / 2.0))// maximum allowable current (half A/D range)
#define MAX_FRE_HZ				300.0					// maximum allowable frequency
#define MAX_OMEGA_R_S			(TWOPIF * MAX_FRE_HZ)	// maximum allowable angular velocity [rad/sec]
#define MAX_OMEGA_RPM			(60.0 * MAX_FRE_HZ)		// maximum allowable angular velocity [rpm]

// Base values definition
#define BASE_VOLTAGE_VOL		(MAX_VOLTAGE_VOL * 1.0)	// multiply the max. volt by 2 or 4 if deep flux weakening
#define BASE_CURRENT_AMP		(MAX_CURRENT_AMP * 1.0)
#define BASE_SPEED_R_S			MAX_OMEGA_R_S
#define BASE_IMPEDANCE_OHM		(BASE_VOLTAGE_VOL / BASE_CURRENT_AMP)
#define BASE_RESISTANCE_OHM		BASE_IMPEDANCE_OHM
#define BASE_INDUCTANCE_HEN		(BASE_IMPEDANCE_OHM / BASE_SPEED_R_S)
#define BASE_FLUX_WEB			(BASE_VOLTAGE_VOL / BASE_SPEED_R_S)

/*
Voltage: the base voltage is represented with the normalized value NORMVAL=16384.
	Voltage[int]=KV*Voltage[Vol], KV=NORMVAL/BASE_VOLTAGE_VOL
*/
#define	KV				(NORMVALF / BASE_VOLTAGE_VOL)		// float!

/*
Current: the base current is represented with the normalized value NORMVAL=16384.
	Current[int]=KI*Current[Amp], KI=NORMVAL/BASE_CURRENT_AMP
*/
#define	KI				(NORMVALF / BASE_CURRENT_AMP)		// float!

/*
Resistance: the base resistance is represented with the normalized value NORMVAL=16384.
	Resistance[int]=KR*Resistance[Ohm], KR=NORMVAL/BASE_RESISTANCE_OHM
*/
#define	KR				(NORMVALF / BASE_RESISTANCE_OHM)		// float!

/*
Inductance: the base inductance is represented with the normalized value NORMVAL=16384.
	Inductance[int]=KL*Inductance[Henry], KL=NORMVAL/BASE_INDUCTANCE_HEN
Warning: KL can be (much) higher than 32768: test KL_LOW
*/
#define	KL				(NORMVALF / BASE_INDUCTANCE_HEN)		// float!
#define SH_KL_LOW		( 7 )
#define KL_LOW			(KL / ((1 << SH_KL_LOW)))

/*
Flux: the base flux is represented with the normalized value NORMVAL=16384.
	Flux[int]=KF*Flux[weber], KF=NORMVAL/BASE_FLUX_WEBER
Warning: KF can be (much) higher than 32768: test KF_LOW
*/
#define	KF				(NORMVALF / BASE_FLUX_WEB)			// float!
#define SH_KF_LOW		( 5 )
#define KF_LOW			(KF / ((1 << SH_KF_LOW)))

/*
Time: the time is expressed as a multiple of the sampling period Ts.
	Time[int]=KT*Time[sec], KT=Fs (Fs=sampling_frequency=1/Ts)
*/
#define	KT				SAM_FRE_HZ							// float!

/*
Constant used in flux integration KX=(KT*KV)/KF
Warning: for resolution reasons, we should amplify this constant (verify the results!)
*/
#define KX				(KT * KV / KF)						// float!
#define SH_KX_AMP		( 12 )
#define AMP_KX			(1 << SH_KX_AMP)
#define KX_AMP			(AMP_KX * KX)

/*
Angles: the interval [0, 2pi) is represented with the interval [0, 65536), with the resolution of
2pi/65536 rad.
	Angle[int]=KAF*Angle[rad], KAF=32768/pi(=10430.37835)
Note that in this way the angle can be considered unsigned in the range [0, 65536), or signed in
the range [-32768, 32768), with identical results. In every case the representation requires a
16bit word.
*/
#ifndef ANGLES
#define ANGLES
// angles definitions
#define	PISIXTHS	 	5461	// 0x1555 030
#define	PIFOURTHS		8192	// 0x2000 045
#define	PITHIRDS		10923	// 0x2AAB 060
#define	PIHALVES		16384	// 0x4000 090
#define	TWOPITHIRDS		21845	// 0x5555 120
#define	THREEPIFOURTHS	24576	// 0x6000 135
#define	FIVEPISIXTHS	27307	// 0x6AAB 150
#define	PI				32768	// 0x8000 180
#define	SEVENPISIXTHS	38229	// 0x9555 210
#define	FIVEPIFOURTHS	40960	// 0xA000 225
#define	FOURPITHIRDS	43691	// 0xAAAB 240
#define	THREEPIHALVES	49152	// 0xC000 270
#define	FIVEPITHIRDS	54613	// 0xD555 300
#define	SEVENPIFOURTHS	57344	// 0xE000 315
#define	ELEVENPISIXTHS	60075	// 0xEAAB 330
#define	TWOPI			65536	// 0x00010000 LONG; IF WORD ACCESS GIVES 0
#define	ANGMOD			0xFFFF	// TWOPI-1
#define	ANG1Q			0x3FFF	// selects first_quarter_value
#define	QUAMOD			0xC000	// selects quarter
#define	ISCOS			0x4000	// table(first_quarter_value) gives cos
#define	ISNEG			0x8000	// sin is neg
// angles conversion constants
#define KAF				(TWOPI / TWOPIF)  // angles conversion constant
#define KA				((uint16_t)KAF)	  // angles conversion constant
// Trigonometric functions: sin(a), cos(a) are normalized to NORMVAL=16384.
#define SH_NORMSIN		SH_NORMVAL
#define NORMSIN			((int16_t)( 1 << SH_NORMSIN ))
#define SH_NORMTAN		SH_NORMVAL
#define NORMTAN			((int16_t)( 1 << SH_NORMTAN ))
#endif	// ANGLES

/*
Angular velocity: it is expressed as a function of angles and time, in order to obtain the angular
velocity as the subtractions of two angles in two sampling moments; for resolution reasons, an
amplification is needed, and we choose this amplification equal to NORMVAL=16384.
	Omega[in1]=KO1*Omega[r/s]=(KO1*KT/KAF)*Angle[int]/Time[int]
Since we want:
	Omega[in1]=NORMVAL*Angle[int]/Time[int]
	( -> Omega[in1]=NORMVAL*(Angle(n)-Angle(n-k))/k )
We obtain:
	KO1=NORMVAL*KAF/KT=NORMVAL*65536/(2*pi*Fs)
The entire speed range cannot, in general, be represented in a 16bit word, but a long is needed.
This high resolution is needed only in calculation involved in angles management, while when the
speed is used to calculate voltages, lesser resolution is enough.
So the second representation of the angular speed is the following:
	Omega[in2]=KO2*Omega[r/s], KO2=NORMVAL/BASE_SPEED_R_S
To pass from a representation to the other we have the following relationship:
	Omega[in2]/KO2=Omega[r/s]=Omega[in1]/KO1 -> Omega[in2]=(KO2/KO1)Omega[in1]
	KO2/KO1=(NORMVAL/MAX_SPEED_R_S)/(NORMVAL*65536/(2*pi*Fs))=(2*pi*Fs)/(65536*BASE_SPEED_R_S)
	KO1/KO2=65536*MAX_FRE_HZ/Fs
*/
#define	KO1				(NORMVALF * KAF / KT)		// float!
#define KO2				(NORMVALF / BASE_SPEED_R_S)	// float!
#define KO1_2			(KO1 / KO2)					// float!

/* some speed values */
#define OM2_1HZF		(TWOPIF * KO2)
#define OM2_HZ( X )		((uint16_t)(OM2_1HZF * X))
#define OM2_1HZ			((uint16_t)OM2_1HZF)
#define OM2_2HZ			OM2_HZ( 2 )					// example

/*
Calculation relations: due to the internal representation choice, the following relations are valid:
	Omega[in2]=Omega[in1]/((uint16_t) KO1_2);
	Impedance[int]=Inductance[int]*Omega[in2]/NORMVAL
	Flux[int]=Inductance[int]*Current[int]/NORMVAL
	Voltage[int]=Impedance[int]*Current[int]/NORMVAL
    Voltage[int]=Flux[int]*Omega[in2]/NORMVAL
*/

/*
Other useful relations:
	frequency[Hz]=omega[rad/sec]/(2*pi)=omega[int2]/(2*pi*KO2)=omega[int2]*MAX_FRE_HZ/NORMVAL
	mechanical_angular_velocity[rpm]=60*electro_mechanical_velocity[rad/sec]/(poles_pair_number*2*pi)
									=(electro_mechanical_velocity[int2]/poles_pair_number)*(60/(2*pi*KO2))
									=(electro_mechanical_velocity[int2]/poles_pair_number)*(MAX_OMEGA_RPM/NORMVAL)
									=(electro_mechanical_velocity[int2]/poles_pair_number)*NORMVAL/KRPMO
*/
#define KOF				MAX_FRE_HZ		// frequency[Hz]=omega[int2]*MAX_FRE_HZ/NORMVAL
#define KOF10			(KOF * 10)		// freq[Hz/10]=omega[int2]*((uint16_t)KOF10)/NORMVAL
#define KORPM			MAX_OMEGA_RPM	// omega[rpm]=omega[int2]*MAX_OMEGA_RPM/NORMVAL
#define KRPMO			(NORMVALF * NORMVALF / MAX_OMEGA_RPM)	// omega[int2]=omega[rpm]*((uint16_t)KRPMO)/NORMVAL

// currents A/D conversion constant
//#define KADIF			(2.0 * NORMVALF / ADFS)	// Current[int]=KI*Current[amp]=KI*RAV_AD*(ADVCC/ADFS)*(AD_READING-AD_OFFSET)
// note: since KI=(NORMVAL/BASE_CURRENT_AMP), and BASE_CURRENT_AMP=(RAV_AD*ADVCC/2.0), we have:
// KADI=KI*RAV_AD*(ADVCC/ADFS)=(NORMVAL*2.0/(RAV_AD*ADVCC))*RAV_AD*(ADVCC/ADFS)=(2.0*NORMVAL/ADFS)
// Warning: KADI should be a (small) integer (power of two).
// Amplification and compensation with the reference voltage reading is then made, and the
// conversion will be:
// kadi = div_w(KADICOMP, vreff);
// asl = mul_w(ad_reading, kadi);
// current = asl >> SH_KADI;
#ifdef CUR_REV
#define KADIF			(-(KI * RAV_AD * ADVCC) / ADFS)	// Current[int]=KI*Current[amp]=KI*RAV_AD*(ADVCC/ADFS)*(AD_READING-AD_OFFSET)
#else	// CUR_REV
#define KADIF			((KI * RAV_AD * ADVCC) / ADFS)	// Current[int]=KI*Current[amp]=KI*RAV_AD*(ADVCC/ADFS)*(AD_READING-AD_OFFSET)
#endif	// CUR_REV
#define SH_KADI			( 10 )
#define AMP_KADI		(1 << SH_KADI)
#define KADI			((int16_t)(AMP_KADI * KADIF))

// voltages (bus and output) A/D conversion constant
//#define KADVF			((NORMVALF / ADFS)*(MAX_VOLTAGE_VOL / BASE_VOLTAGE_VOL))		// Voltage[int]=KV*Voltage[vol]=KV*RVV_AD*(ADVCC/ADFS)*AD_READING
// note: since KV=(NORMVAL/BASE_VOLTAGE_VOL), and MAX_VOLTAGE_VOL=(RVV_AD*ADVCC), we have:
// KADV=KV*RVV_AD*(ADVCC/ADFS)*(MAX_VOLTAGE_VOL/BASE_VOLTAGE_VOL)=(NORMVAL/(RVV_AD*ADVCC))*RVV_AD*
// *(ADVCC/ADFS)*(MAX_VOLTAGE_VOL/BASE_VOLTAGE_VOL)=(NORMVAL/ADFS)*(MAX_VOLTAGE_VOL/BASE_VOLTAGE_VOL)
// Warning: KADV should be a (small) integer (power of two).
// Amplification and compensation with the reference voltage reading is then made, and the
// conversion will be:
// kadv = div_w(KADVCOMP, vreff);
// asl = mul_w(ad_reading, kadv);
// v_bus = asl >> SH_KADI;
#define KADVF			((KV * RVV_AD * ADVCC) / ADFS)	// Voltage[int]=KV*Voltage[vol]=KV*RVV_AD*(ADVCC/ADFS)*AD_READING
#define SH_KADV			( 10 )
#define AMP_KADV		(1 << SH_KADV)
#define KADV			((int16_t)(AMP_KADV * KADVF))

// startup
#define STARTUP_VAL		(CNT_MS(STUP_SPE_RT_MS))

// trip control
#define TRIP_CONTROL
#define MAX_TRIP		( 50 )

// alarms
#define	EQP_ALL			( 1 )	// eeprom alarm code
#define	FAULT_ALL		( 2 )	// ipm hardware alarm code
#define	TRIP_ALL		( 3 )	// loss of phase alarm code

#endif // _CONST_DEF_H


