
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		hardware_wsetup.c
*	Project:	MCRP09_RX200 (ss) (YRotateItRX200_1s_V1)
*	Version:	1.0
*	Date:		19-11-2013 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	Hardware setup file
*******************************************************************************/

#include "globdef.h"

/* PIN DESCRIPTION YRMCKITRX220 ************************************************

 1 VCL
 2 MD			emu
 3 RES			res
 4 P37 XTAL		xtal	IN
 5 VSS
 6 P36 EXTAL	extal	OUT
 7 VCC
 8 P35 NMI		IO2		IN
 9 P31			EEPSCL	OUT
10 P30			EEPSDA	I/O
11 P27 MTIOC2B	ANOUT2	OUT
12 P26 MTIOC2A	ANOUT1	OUT
>
13 P17			IO1		IN
14 P16 TXD1		TX		OUT
15 P17 RXD1		RX		IN
16 P14			KEY		IN
17 PH3			DL3		OUT
18 PH2			DL2		OUT
19 PH1			LIM_OFF	OUT
20 PH0			EXT_SUP	IN
21 PC7 TCKLB	ENCB	IN
22 PC6 TCKLA	ENCA	IN
23 PC5 MTIOC3B	PWM_UH	OUT
24 PC4 MTIOC3D	PWM_UL	OUT
>
25 PC3 POE0		OVC		IN
26 PC2 MTIOC4A	PWM_VH	OUT
27 PC1 MTIOC4C	PWM_VL	OUT
28 VCC
29 PC0 MTIC5W	HALLW	IN
30 VSS
31 PA6 MTIC5V	HALLV	IN
32 PA4 MTIC5U	HALLU	IN
33 PA3 IRQ6		ENCZ	IN
34 PA1 MTIOC0B	ANOUT3	OUT
35 PE4 MTIOC4D	PWM_WL	OUT
36 PE3 MTIOC4B	PWM_WH	OUT
>
37 PE2 AN10		REF		IN
38 PE1 AN9		TMP		IN
39 NC
40 P46 AN6		VB		IN
41 NC
42 P42 AN2		IW		IN
43 P41 AN1		IV		IN
44 VREFL
45 P40 AN0		IU		IN
46 VREFH
47 AVCC
48 AVSS

*******************************************************************************/

#define DEF_P0PDR	0xA8	// non-existent pin: P03, P05, P07
#define DEF_P1PDR	0x0C	// non-existent pin: P12, P13
#define DEF_P2PDR	0x3F	// non-existent pin: P20 to P25
#define DEF_P3PDR	0x1C	// non-existent pin: P32 to P34
#define DEF_P4PDR	0xB8	// non-existent pin: P43 to P45, P47
#define DEF_P5PDR	0x3F	// non-existent pin: P50 to P55
#define DEF_PAPDR	0xA5	// non-existent pin: PA0, PA2, PA5, PA7
#define DEF_PBPDR	0xD4	// non-existent pin: PB2, PB4, PB6, PB7
#define DEF_PCPDR	0x0F	// non-existent pin: PC0 to PC3
#define DEF_PDPDR	0xFF	// non-existent pin: PD0 to PD7
#define DEF_PEPDR	0xE1	// non-existent pin: PE0, PE5 to PE7
//#define DEF_PHPDR	0x00	// non-existent pin: none
#define DEF_PJPDR	0x0A	// non-existent pin: PJ1, PJ3

void HardwareSetup(void)
{
	uint32_t i, dummy;

	// nonexistent port init as output
	PORT0.PDR.BYTE = PORT0.PDR.BYTE | DEF_P0PDR;
	PORT1.PDR.BYTE = PORT1.PDR.BYTE | DEF_P1PDR;
	PORT2.PDR.BYTE = PORT2.PDR.BYTE | DEF_P2PDR;
	PORT3.PDR.BYTE = PORT3.PDR.BYTE | DEF_P3PDR;
	PORT4.PDR.BYTE = PORT4.PDR.BYTE | DEF_P4PDR;
	PORT5.PDR.BYTE = PORT5.PDR.BYTE | DEF_P5PDR;
	PORTA.PDR.BYTE = PORTA.PDR.BYTE | DEF_PAPDR;
	PORTB.PDR.BYTE = PORTB.PDR.BYTE | DEF_PBPDR;
	PORTC.PDR.BYTE = PORTC.PDR.BYTE | DEF_PCPDR;
	PORTD.PDR.BYTE = PORTD.PDR.BYTE | DEF_PDPDR;
	PORTE.PDR.BYTE = PORTE.PDR.BYTE | DEF_PEPDR;
//	PORTH.PDR.BYTE = PORTH.PDR.BYTE | DEF_PHPDR;
	PORTJ.PDR.BYTE = PORTJ.PDR.BYTE | DEF_PJPDR;


	// CLOCK CIRCUIT INIT (subclock disable, HOCO enable and setting)

	SYSTEM.PRCR.WORD = 0xA503;				// enable writing PRC0, PRC1

	// subclock disable
    SYSTEM.SOSCCR.BYTE = 0x01;
    while(0x01 != SYSTEM.SOSCCR.BYTE);		// Confirm that the written value can be read correctly
    RTC.RCR3.BIT.RTCEN = 0;					// Sub-clock oscillator is stopped
    for(i=0; i<3; i++)
        dummy = RTC.RCR3.BIT.RTCEN;			// dummy read three times
    while(0 != RTC.RCR3.BIT.RTCEN);			// Confirm that the written value can be read correctly

	// HOCO enable and setting
    SYSTEM.HOCOCR2.BYTE = 0x00;				// set HOCO frequency 32MHz
    SYSTEM.HOCOWTCR2.BYTE = 0x14;			// 180 cycles stabilization time
    SYSTEM.HOCOCR.BYTE = 0x00;				// operate the HOCO
	DELAY1US( 50 )							// wait for stabilization (at least 20us)
	SYSTEM.SCKCR.LONG = 0x00011010;			// internal clocks division ratio (no division for all except BCK)
    while(0x00011010!=SYSTEM.SCKCR.LONG);	// Confirm that the written value can be read correctly
    SYSTEM.SCKCR3.WORD = 0x0100;			// select HOCO as internal clock source
    while(0x0100 != SYSTEM.SCKCR3.WORD);	// Confirm that the written value can be read correctly
    //SYSTEM.OPCCR.BYTE = 0x02;				// middle-speed operation 1A (default)
    //while(0 != SYSTEM.OPCCR.BIT.OPCMTSF);	// Confirm that the operation power control mode transition completed

    SYSTEM.PRCR.WORD = 0xA500;				// disable writing PRC0, PRC1


	// Unused modules stop
    SYSTEM.PRCR.WORD = 0xA502;			// enable writing PRC1
    MSTP(DTC) = 1;              		// DMAC/DTC trans to module-stop state
    SYSTEM.PRCR.WORD = 0xA500;			// disable writing PRC1


	// PORT B/C SWITCHING	*******

	//PORT.PSRA.BYTE = 0x00;
	//PORT.PSRB.BIT.PSEL7 = 0;		// B7	1 -> PIN27 becomes PC1 not PB7
	//PORT.PSRB.BIT.PSEL6 = 0;		// B6	1 -> PIN28 becomes PC0 not PB6
		// B5, .., B0

	//PORT.PSRB.BYTE = 0x00;
		// B7, B6
	//PORT.PSRB.BIT.PSEL5 = 0;		// B5	1 -> PIN25 becomes PC3 not PB5
		// B4
	//PORT.PSRB.BIT.PSEL3 = 0;		// B3	1 -> PIN26 becomes PC2 not PB3
		// B2
	//PORT.PSRB.BIT.PSEL1 = 0;		// B1	1 -> PIN27 becomes PC1 not PB1
	//PORT.PSRB.BIT.PSEL0 = 0;		// B0	1 -> PIN28 becomes PC0 not PB0


	// PORT1	********************

	PORT1.PDR.BYTE |= 0x40;			// port direction register
	//PORT1.PDR.BIT.B7 = 0;			// PIN13 P17		IN  IO1
	//PORT1.PDR.BIT.B6 = 1;			// PIN14 P16 TXD1	OUT TX
	//PORT1.PDR.BIT.B5 = 0;			// PIN15 P15 RXD1	IN  RX
	//PORT1.PDR.BIT.B4 = 0;			// PIN16 P14		IN  KEY
		// B3, .., B0

	PORT1.PODR.BYTE = 0x40;			// port output data register
	//PORT1.PODR.BIT.B7 = 0;		// PIN13 P17		IN  IO1
	//PORT1.PODR.BIT.B6 = 1;		// PIN14 P16 TXD1	OUT TX
	//PORT1.PODR.BIT.B5 = 0;		// PIN15 P15 RXD1	IN  RX
	//PORT1.PODR.BIT.B4 =0;			// PIN16 P14		IN  KEY
		// B3, .., B0

	//PORT1.PIDR.BYTE				// port input data register
	//PORT1.PIDR.BIT.B7				// PIN13 P17		IN  IO1
	//PORT1.PIDR.BIT.B6				// PIN14 P16 TXD1	OUT TX
	//PORT1.PIDR.BIT.B5				// PIN15 P15 RXD1	IN  RX
	//PORT1.PIDR.BIT.B4				// PIN16 P14		IN  KEY
		// B3, .., B0

	//PORT1.PMR.BYTE = 0x00;		// port mode register
	//PORT1.PMR.BIT.B7 = 0;			// PIN13 P17		IN  IO1
	//PORT1.PMR.BIT.B6 = 0;			// PIN14 P16 TXD1	OUT TX
	//PORT1.PMR.BIT.B5 = 0;			// PIN15 P15 RXD1	IN  RX
	//PORT1.PMR.BIT.B4 = 0;			// PIN16 P14		IN  KEY
		// B3, .., B0

	//PORT1.ODR0.BYTE = 0x00;		// open drain control register 0
	//PORT1.ODR1.BYTE = 0x00;		// open drain control register 1
	//PORT1.PCR.BYTE = 0x00;		// pull up control register
	//PORT1.DSCR.BYTE = 0x00;		// drive capacity control register


	// PORT2	********************

	PORT2.PDR.BYTE |= 0xC0;			// port direction register
	//PORT2.PDR.BIT.B7 = 1;			// PIN11 P27 MTIOC2B	OUT ANOUT2
	//PORT2.PDR.BIT.B6 = 1;			// PIN12 P26 MTIOC2A	OUT ANOUT1
		// B5, .., B0

	//PORT2.PODR.BYTE = 0x00;		// port output data register
	//PORT2.PODR.BIT.B7 = 0;		// PIN11 P27 MTIOC2B	OUT ANOUT2
	//PORT2.PODR.BIT.B6 = 0;		// PIN12 P26 MTIOC2A	OUT ANOUT1
		// B5, .., B0

	//PORT2.PIDR.BYTE				// port input data register
	//PORT2.PIDR.BIT.B7				// PIN11 P27 MTIOC2B	OUT ANOUT2
	//PORT2.PIDR.BIT.B6				// PIN12 P26 MTIOC2A	OUT ANOUT1
		// B5, .., B0

	//PORT2.PMR.BYTE = 0x00;		// port mode register
	//PORT2.PMR.BIT.B7 = 0;			// PIN11 P27 MTIOC2B	OUT ANOUT2
	//PORT2.PMR.BIT.B6 = 0;			// PIN12 P26 MTIOC2A	OUT ANOUT1
		// B5, .., B0

	//PORT2.ODR1.BYTE = 0x00;		// open drain control register 1
	//PORT2.PCR.BYTE = 0x00;		// pull up control register


	// PORT3	********************

	PORT3.PDR.BYTE |= 0x42;			// port direction register
	//PORT3.PDR.BIT.B7 = 0;			// PIN4  P37 XTAL	IN
	//PORT3.PDR.BIT.B6 = 1;			// PIN6  P36 EXTAL	OUT
		// B5, .., B2
	//PORT3.PDR.BIT.B1 = 1;			// PIN9  P31		OUT EEPSCL
	//PORT3.PDR.BIT.B0 = 0;			// PIN10 P30		I/O EEPSDA

	PORT3.PODR.BYTE = 0x02;			// port output data register
	//PORT3.PODR.BIT.B7 = 0;		// PIN4  P37 XTAL	IN
	//PORT3.PODR.BIT.B6 = 0;		// PIN6  P36 EXTAL	OUT
		// B5, .., B2
	//PORT3.PODR.BIT.B1 = 1;		// PIN9  P31		OUT EEPSCL
	//PORT3.PODR.BIT.B0 = 0;		// PIN10 P30		I/O EEPSDA

	//PORT3.PIDR.BYTE				// port input data register
	//PORT3.PIDR.BIT.B7				// PIN4  P37 XTAL	IN
	//PORT3.PIDR.BIT.B6				// PIN6  P36 EXTAL	OUT
	//PORT3.PIDR.BIT.B5				// PIN8  P35 NMI	IN  IO2
		// B4, .., B2
	//PORT3.PIDR.BIT.B1				// PIN9  P31		OUT EEPSCL
	//PORT3.PIDR.BIT.B0				// PIN10 P30		I/O EEPSDA

	//PORT3.PMR.BYTE = 0x00;		// port mode register
		// B7, .., B2
	//PORT3.PMR.BIT.B1 = 0;			// PIN9  P31		OUT EEPSCL
	//PORT3.PMR.BIT.B0 = 0;			// PIN10 P30		I/O EEPSDA

	PORT3.ODR0.BYTE = 0x01;			// open drain control register 
		// B7
	//PORT3.ODR0.BIT.B6 = 0;		//
		// B7
	//PORT3.ODR0.BIT.B4 = 0;		//
		// B3, B2, B1
	//PORT3.ODR0.BIT.B0 = 1;		// PIN10 P30 I/O EEPSDA 1 -> open drain output

	//PORT3.ODR1.BYTE = 0x00;		// open drain control register 1
	//PORT3.PCR.BYTE = 0x00;		// pull up control register


	// PORT4	********************

	//PORT4.PDR.BYTE |= 0x00;		// port direction register
		// B7
	//PORT4.PDR.BIT.B6 = 0;			// PIN40 P46 AN6	IN  VB
		// B5, .., B3
	//PORT4.PDR.BIT.B2 = 0;			// PIN42 P42 AN2	IN  IW
	//PORT4.PDR.BIT.B1 = 0;			// PIN43 P41 AN1	IN  IV
	//PORT4.PDR.BIT.B0 = 0;			// PIN45 P40 AN0	IN  IU

	//PORT4.PODR.BYTE = 0x00;		// port output data register
		// B7
	//PORT4.PODR.BIT.B6 = 0;		// PIN40 P46 AN6	IN  VB
		// B5, .., B3
	//PORT4.PODR.BIT.B2 = 0;		// PIN42 P42 AN2	IN  IW
	//PORT4.PODR.BIT.B1 = 0;		// PIN43 P41 AN1	IN  IV
	//PORT4.PODR.BIT.B0 = 0;		// PIN45 P40 AN0	IN  IU

	//PORT4.PIDR.BYTE				// port input data register
		// B7
	//PORT4.PIDR.BIT.B6				// PIN40 P46 AN6	IN  VB
		// B5, .., B3
	//PORT4.PIDR.BIT.B2				// PIN42 P42 AN2	IN  IW
	//PORT4.PIDR.BIT.B1				// PIN43 P41 AN1	IN  IV
	//PORT4.PIDR.BIT.B0				// PIN45 P40 AN0	IN  IU

	//PORT4.PMR.BYTE = 0x00;		// port mode register
		// B7
	//PORT4.PMR.BIT.B6 = 0;			// PIN40 P46 AN6	IN  VB
		// B5, .., B3
	//PORT4.PMR.BIT.B2 = 0;			// PIN42 P42 AN2	IN  IW
	//PORT4.PMR.BIT.B1 = 0;			// PIN43 P41 AN1	IN  IV
	//PORT4.PMR.BIT.B0 = 0;			// PIN45 P40 AN0	IN  IU

	//PORT4.PCR.BYTE = 0x00;		// pull up control register


	// PORTA	********************

	PORTA.PDR.BYTE |= 0x02;			// port direction register
		// B7
	//PORTA.PDR.BIT.B6 = 0;			// PIN31 PA6 MTIC5V		IN  HALLV
		// B5
	//PORTA.PDR.BIT.B4 = 0;			// PIN32 PA4 MTIC5U		IN  HALLU
	//PORTA.PDR.BIT.B3 = 0;			// PIN33 PA3 IRQ6		IN  ENCZ
		// B2
	//PORTA.PDR.BIT.B1 = 1;			// PIN34 PA1 MTIOC0B	OUT ANOUT3
		// B0

	//PORTA.PODR.BYTE = 0x00;		// port output data register
		// B7
	//PORTA.PODRBIT.B6 = 0;			// PIN31 PA6 MTIC5V		IN  HALLV
		// B5
	//PORTA.PODR.BIT.B4 = 0;		// PIN32 PA4 MTIC5U		IN  HALLU
	//PORTA.PODR.BIT.B3 = 0;		// PIN33 PA3 IRQ6		IN  ENCZ
		// B2
	//PORTA.PODR.BIT.B1 = 0;		// PIN34 PA1 MTIOC0B	OUT ANOUT3
		// B0

	//PORTA.PIDR.BYTE				// port input data register
		// B7
	//PORTA.PIDR.BIT.B6				// PIN31 PA6 MTIC5V		IN  HALLV
		// B5
	//PORTA.PIDR.BIT.B4				// PIN32 PA4 MTIC5U		IN  HALLU
	//PORTA.PIDR.BIT.B3				// PIN33 PA3 IRQ6		IN  ENCZ
		// B2
	//PORTA.PIDR.BIT.B1				// PIN34 PA1 MTIOC0B	OUT ANOUT3
		// B0

	//PORTA.PMR.BYTE |= 0x00;		// port mode register
		// B7
	//PORTA.PMR.BIT.B6 = 0;			// PIN31 PA6 MTIC5V		IN  HALLV
		// B5
	//PORTA.PMR.BIT.B4 = 0;			// PIN32 PA4 MTIC5U		IN  HALLU
	//PORTA.PMR.BIT.B3 = 0;			// PIN33 PA3 IRQ6		IN  ENCZ
		// B2
	//PORTA.PMR.BIT.B1 = 0;			// PIN34 PA1 MTIOC0B	OUT ANOUT3
		// B0

	//PORTA.ODR0.BYTE |= 0x00;		// open drain control register 0
	//PORTA.ODR1.BYTE |= 0x00;		// open drain control register 1
	//PORTA.PCR.BYTE |= 0x00;		// pull up control register


	// PORTB	********************
	// WARNING: PWM OUTPUTS ARE HIGH ACTIVE!

	PORTB.PODR.BYTE = 0x00;		// port output data register
	PORTB.PDR.BYTE |= 0x0A;		// port direction register
		// B7, B6
	//PORTB.PDR.BIT.B5 = 0;			// PIN25 PB5 (PC3) POE0		IN  OVC
		// B4
	//PORTB.PDR.BIT.B3 = 1;			// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
		// B2
	//PORTB.PDR.BIT.B1 = 1;			// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTB.PDR.BIT.B0 = 0;			// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	PORTB.PODR.BYTE = 0x00;		// port output data register
		// B7, B6
	//PORTB.PODR.BIT.B5 = 0;		// PIN25 PB5 (PC3) POE0		IN  OVC
		// B4
	//PORTB.PODR.BIT.B3 = 0;		// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
		// B2
	//PORTB.PODR.BIT.B1 = 0;		// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTB.PODR.BIT.B0 = 0;		// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTB.PIDR.BYTE				// port input data register
		// B7, B6
	//PORTB.PIDR.BIT.B5				// PIN25 PB5 (PC3) POE0		IN  OVC
		// B4
	//PORTB.PIDR.BIT.B3				// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
		// B2
	//PORTB.PIDR.BIT.B1				// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTB.PIDR.BIT.B0				// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTB.PMR.BYTE = 0x00;		// port mode register
		// B7, B6
	//PORTB.PMR.BIT.B5 = 0;			// PIN25 PB5 (PC3) POE0		IN  OVC
		// B4
	//PORTB.PMR.BIT.B3 = 0;			// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
		// B2
	//PORTB.PMR.BIT.B1 = 0;			// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTB.PMR.BIT.B0 = 0;			// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTB.ODR0.BYTE = 0x00;		// open drain control register 0
	//PORTB.ODR1.BYTE = 0x00;		// open drain control register 1
	//PORTB.PCR.BYTE = 0x00;		// pull up control register
	//PORTB.DSCR.BYTE = 0x00;		// drive capacity control register


	// PORTC	********************
	// WARNING: PWM OUTPUTS ARE HIGH ACTIVE!

	PORTC.PODR.BYTE = 0x00;		// port output data register
	PORTC.PDR.BYTE |= 0x30;		// port direction register
	//PORTC.PDR.BIT.B7 = 0;			// PIN21 PC7 TCKLB			IN  ENCB
	//PORTC.PDR.BIT.B6 = 0;			// PIN22 PC6 TCKLA			IN  ENCA
	//PORTC.PDR.BIT.B5 = 1;			// PIN23 PC5 MTIOC3B		OUT PWM_UH
	//PORTC.PDR.BIT.B4 = 1;			// PIN24 PC4 MTIOC3D		OUT PWM_UL
	//PORTC.PDR.BIT.B3 = 0;			// PIN25 PB5 (PC3) POE0		IN  OVC
	//PORTC.PDR.BIT.B2 = 0;			// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
	//PORTC.PDR.BIT.B1 = 0;			// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTC.PDR.BIT.B0 = 0;			// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	PORTC.PODR.BYTE = 0x00;		// port output data register
	//PORTC.PODR.BIT.B7 = 0;		// PIN21 PC7 TCKLB			IN  ENCB
	//PORTC.PODR.BIT.B6 = 0;		// PIN22 PC6 TCKLA			IN  ENCA
	//PORTC.PODR.BIT.B5 = 0;		// PIN23 PC5 MTIOC3B		OUT PWM_UH
	//PORTC.PODR.BIT.B4 = 0;		// PIN24 PC4 MTIOC3D		OUT PWM_UL
	//PORTC.PODR.BIT.B3 = 0;		// PIN25 PB5 (PC3) POE0		IN  OVC
	//PORTC.PODR.BIT.B2 = 0;		// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
	//PORTC.PODR.BIT.B1 = 0;		// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTC.PODR.BIT.B0 = 0;		// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTC.PIDR.BYTE				// port input data register
	//PORTC.PIDR.BIT.B7				// PIN21 PC7 TCKLB			IN  ENCB
	//PORTC.PIDR.BIT.B6				// PIN22 PC6 TCKLA			IN  ENCA
	//PORTC.PIDR.BIT.B5				// PIN23 PC5 MTIOC3B		OUT PWM_UH
	//PORTC.PIDR.BIT.B4				// PIN24 PC4 MTIOC3D		OUT PWM_UL
	//PORTC.PIDR.BIT.B3				// PIN25 PB5 (PC3) POE0		IN  OVC
	//PORTC.PIDR.BIT.B2				// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
	//PORTC.PIDR.BIT.B1				// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTC.PIDR.BIT.B0				// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTC.PMR.BYTE =0x00;			// port mode register
	//PORTC.PMR.BIT.B7 = 0;			// PIN21 PC7 TCKLB			IN  ENCB
	//PORTC.PMR.BIT.B6 = 0;			// PIN22 PC6 TCKLA			IN  ENCA
	//PORTC.PMR.BIT.B5 = 0;			// PIN23 PC5 MTIOC3B		OUT PWM_UH
	//PORTC.PMR.BIT.B4 = 0;			// PIN24 PC4 MTIOC3D		OUT PWM_UL
	//PORTC.PMR.BIT.B3 = 0;			// PIN25 PB5 (PC3) POE0		IN  OVC
	//PORTC.PMR.BIT.B2 = 0;			// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
	//PORTC.PMR.BIT.B1 = 0;			// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
	//PORTC.PMR.BIT.B0 = 0;			// PIN29 PB0 (PC0) MTIC5W	IN  HALLW

	//PORTC.ODR0.BYTE = 0x00;		// open drain control register 0
	//PORTC.ODR1.BYTE = 0x00;		// open drain control register 1
	//PORTC.PCR.BYTE = 0x00;		// pull up control register
	//PORTC.DSCR.BYTE = 0x00;		// drive capacity control register


	// PORTE	********************
	// WARNING: PWM OUTPUTS ARE HIGH ACTIVE!

	PORTE.PODR.BYTE = 0x00;		// port output data register
	PORTE.PDR.BYTE |= 0x18;		// port direction register
		// B7, .., B5
	//PORTE.PDR.BIT.B4 = 1;			// PIN35 PE4 MTIOC4D	OUT PWM_WL
	//PORTE.PDR.BIT.B3 = 1;			// PIN36 PE3 MTIOC4B	OUT PWM_WH
	//PORTE.PDR.BIT.B2 = 0;			// PIN37 PE2 AN10		IN  REF
	//PORTE.PDR.BIT.B1 = 0;			// PIN38 PE1 AN9		IN  TMP
		// B0

	PORTE.PODR.BYTE = 0x00;		// port output data register
		// B7, .., B5
	//PORTE.PODR.BIT.B4 = 0;		// PIN35 PE4 MTIOC4D	OUT PWM_WL
	//PORTE.PODR.BIT.B3 = 0;		// PIN36 PE3 MTIOC4B	OUT PWM_WH
	//PORTE.PODR.BIT.B2 = 0;		// PIN37 PE2 AN10		IN  REF
	//PORTE.PODR.BIT.B1 = 0;		// PIN38 PE1 AN9		IN  TMP
		// B0

	//PORTE.PIDR.BYTE				// port input data register
		// B7, .., B5
	//PORTE.PIDR.BIT.B4				// PIN35 PE4 MTIOC4D	OUT PWM_WL
	//PORTE.PIDR.BIT.B3				// PIN36 PE3 MTIOC4B	OUT PWM_WH
	//PORTE.PIDR.BIT.B2				// PIN37 PE2 AN10		IN  REF
	//PORTE.PIDR.BIT.B1				// PIN38 PE1 AN9		IN  TMP
		// B0

	//PORTE.PMR.BYTE = 0x00;		// port mode register
		// B7, .., B5
	//PORTE.PMR.BIT.B4 = 0;			// PIN35 PE4 MTIOC4D	OUT PWM_WL
	//PORTE.PMR.BIT.B3 = 0;			// PIN36 PE3 MTIOC4B	OUT PWM_WH
	//PORTE.PMR.BIT.B2 = 0;			// PIN37 PE2 AN10		IN  REF
	//PORTE.PMR.BIT.B1 = 0;			// PIN38 PE1 AN9		IN  TMP
		// B0

	//PORTE.ODR0.BYTE = 0x00;		// open drain control register 0
	//PORTE.ODR1.BYTE = 0x00;		// open drain control register 1
	//PORTE.PCR.BYTE = 0x00;		// pull up control register


	// PORTH	********************

	PORTH.PDR.BYTE |= 0x0E;			// port direction register
		// B7, .., B4
	//PORTH.PDR.BIT.B3 = 1;			// PIN17 PH3	OUT DL3
	//PORTH.PDR.BIT.B2 = 1;			// PIN18 PH2	OUT DL2
	//PORTH.PDR.BIT.B1 = 1;			// PIN19 PH1	OUT LIM_OFF
	//PORTH.PDR.BIT.B0 = 0;			// PIN20 PH0	IN  EXT_SUP

	PORTH.PODR.BYTE = 0x00;			// port output data register
		// B7, .., B4
	//PORTH.PODR.BIT.B3 = 0;		// PIN17 PH3	OUT DL3
	//PORTH.PODR.BIT.B2 = 0;		// PIN18 PH2	OUT DL2
	//PORTH.PODR.BIT.B1 = 0;		// PIN19 PH1	OUT LIM_OFF
	//PORTH.PODR.BIT.B0 = 0;		// PIN20 PH0	IN  EXT_SUP

	//PORTH.PIDR.BYTE				// port input data register
		// B7, .., B4
	//PORTH.PIDR.BIT.B3				// PIN17 PH3	OUT DL3
	//PORTH.PIDR.BIT.B2				// PIN18 PH2	OUT DL2
	//PORTH.PIDR.BIT.B1				// PIN19 PH1	OUT LIM_OFF
	//PORTH.PIDR.BIT.B0				// PIN20 PH0	IN  EXT_SUP

	//PORTH.PMR.BYTE = 0x00;		// port mode register
		// B7, .., B4
	//PORTH.PMR.BIT.B3 = 0;			// PIN17 PH3	OUT DL3
	//PORTH.PMR.BIT.B2 = 0;			// PIN18 PH2	OUT DL2
	//PORTH.PMR.BIT.B1 = 0;			// PIN19 PH1	OUT LIM_OFF
	//PORTH.PMR.BIT.B0 = 0;			// PIN20 PH0	IN  EXT_SUP

	//PORTH.PCR.BYTE = 0x00;		// pull up control register



	// Multi-Function Pin Controller

	//MPC.PWPR.BYTE = 0x00;
	//MPC.PWPR.BIT.B0WI = 0;		// B7
	//MPC.PWPR.BIT.PFSWE = 0;		// B6
		// B5, .., B0


	//MPC.P16PFS.BYTE = 0x00;		// PIN14 P16 TXD1 OUT TX
		// B7
	//MPC.P16PFS.BIT.ISEL = 0;		// B6
		// B5, B4
	//MPC.P16PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.P17PFS.BYTE = 0x00;		// PIN15 P15 RXD1 IN  RX
		// B7
	//MPC.P17PFS.BIT.ISEL = 0;		// B6
		// B5, B4
	//MPC.P17PFS.BIT.PSEL = 0;		// B3, .., B0


	//MPC.P26PFS.BYTE = 0x00;		// PIN12 P26 MTIOC2A OUT ANOUT1
		// B7, .., B4
	//MPC.P26PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.P27PFS.BYTE = 0x00;		// PIN11 P27 MTIOC2B OUT ANOUT2
		// B7, .., B4
	//MPC.P27PFS.BIT.PSEL = 0;		// B3, .., B0


	//MPC.P40PFS.BYTE = 0x00;		// PIN45 P40 AN0 IN  IU
	//MPC.P40PFS.BIT.ASEL = 0;		// B7
		// B6, .., B0

	//MPC.P41PFS.BYTE = 0x00;		// PIN43 P41 AN1 IN  IV
	//MPC.P41PFS.BIT.ASEL = 0;		// B7
		// B6, .., B0

	//MPC.P42PFS.BYTE = 0x00;		// PIN42 P42 AN2 IN  IW
	//MPC.P42PFS.BIT.ASEL = 0;		// B7
		// B6, .., B0

	//MPC.P46PFS.BYTE = 0x00;		// PIN40 P46 AN6 IN  VB
	//MPC.P46PFS.BIT.ASEL = 0;		// B7
		// B6, .., B0


	//MPC.PA1PFS.BYTE = 0x00;		// PIN34 PA1 MTIOC0B OUT ANOUT3
	//MPC.PA1PFS.BIT.ASEL = 0;		// B7
		// B6, .., B4
	//MPC.PA1PFS.BIT.PSEL = 0;		// B3, .., B0


	//MPC.PB1PFS.BYTE = 0x00;		// PIN27 PB1 (PC1) MTIOC4C	OUT PWM_VL
		// B7
	//MPC.PB1PFS.BIT.ISEL = 0;		// B6
		// B5, B4
	//MPC.PB1PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.PB3PFS.BYTE = 0x00;		// PIN26 PB3 (PC2) MTIOC4A	OUT PWM_VH
		// B7, .., B4
	//MPC.PB3PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.PB5PFS.BYTE = 0x00;		// PIN25 PB5 (PC3) POE0		IN  OVC
		// B7, .., B4
	//MPC.PB5PFS.BIT.PSEL = 0;		// B3, .., B0


	//MPC.PC1PFS.BYTE = 0x00;		// PIN27 PC1 MTIOC4C OUT PWM_VL
		// B7, .., B4
	//MPC.PC1PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.PC2PFS.BYTE = 0x00;		// PIN26 PC2 MTIOC4A OUT PWM_VH
		// B7, .., B4
	//MPC.PC2PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.PC3PFS.BYTE = 0x00;		// PIN25 PC3 POE0 IN  OVC
		// B7, .., B4
	//MPC.PC3PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.PC4PFS.BYTE = 0x00;		// PIN24 PC4 MTIOC3D OUT PWM_UL
		// B7, .., B4
	//MPC.PC4PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.PC5PFS.BYTE = 0x00;		// PIN23 PC5 MTIOC3B OUT PWM_UH
		// B7, .., B4
	//MPC.PC5PFS.BIT.PSEL = 0;		// B3, .., B0


	//MPC.PE1PFS.BYTE = 0x00;		// PIN38 PE1 AN9 IN  TMP
	//MPC.PE1PFS.BIT.ASEL = 0;		// B7
		// B6, .., B4
	//MPC.PE1PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.PE2PFS.BYTE = 0x00;		// PIN37 PE2 AN10 IN  REF
	//MPC.PE2PFS.BIT.ASEL = 0;		// B7
		// B6, .., B4
	//MPC.PE2PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.PE3PFS.BYTE = 0x00;		// PIN36 PE3 MTIOC4B	OUT PWM_WH
	//MPC.PE3PFS.BIT.ASEL = 0;		// B7
		// B6, .., B4
	//MPC.PE3PFS.BIT.PSEL = 0;		// B3, .., B0

	//MPC.PE4PFS.BYTE = 0x00;		// PIN35 PE4 MTIOC4D	OUT PWM_WL
	//MPC.PE4PFS.BIT.ASEL = 0;		// B7
		// B6, .., B4
	//MPC.PE4PFS.BIT.PSEL = 0;		// B3, .., B0


}
