
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		pws_E6114.h
*	Project:	MCRP09_RX200 (ss) (YRotateItRX200_1s_V1)
*	Version:	1.0
*	Date:		19-11-2013 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	Internal power stage (E6114A) related definitions
*******************************************************************************/

#ifndef	_PWS_E6114_H
#define _PWS_E6114_H

// current conversion
#define RSHUNT_OHM			0.1 		// shunt value [Ohm]
#define RSGAIN				5.0			// circuit gain

// DC bus voltage conversion
#define RVBUSH_OHM			47000.0		// high side split resistor [Ohm]
#define RVBUSL_OHM			4700.0		// low side split resistor [Ohm]

// deadtimes to be inserted by MTU
#define DEADTIM_NS			900			// deatimes [ns]

// minimum on time of low arms, to guarantee bootstrap operations
// this time should guarantee that the lower switch commutates effectively for enough time to
// charge the bootstrap capacitor
#define MIN_TON_LOW_NS		1500		// [ns] larger than deadtimes

// voltages
#define VB_NOM_VOL			24			// nominal bus voltage [V]

// inverter non linearities compensation
// non-linearities table definitions for MCRP INTERNAL POWER STAGE (E6114A)
// voltage error is INL_O+INL_xx[I]*pwm_freq_Hz*BusVoltage/1000000
#define T_INL_INMAX			0.4
#define T_INL_INDMAX		4
#define INL_TABDIM			32	// greater than T_INL_INDMAX
//#define INL_O_USED
#ifdef INL_O_USED
#define INL_O	0
#endif	// INL_O_USED
#define INL_01	0.4			// 0.1A
#define INL_02	0.53		// 0.2A
#define INL_03	0.57		// 0.3A
#define INL_04	0.585		// 0.4A
#define INL_05	0.585		// 0.5A
#define INL_06	0.585		// 0.6A
#define INL_07	0.585		// 0.7A
#define INL_08	0.585		// 0.8A
#define INL_09	0.585		// 0.9A
#define INL_10	0.585		// 1.0A
#define INL_11	0.585		// 1.1A
#define INL_12	0.585		// 1.2A
#define INL_13	0.585		// 1.3A
#define INL_14	0.585		// 1.4A
#define INL_15	0.585		// 1.5A
#define INL_16	0.585		// 1.6A
#define INL_17	0.585		// 1.7A
#define INL_18	0.585		// 1.8A
#define INL_19	0.585		// 1.9A
#define INL_20	0.585		// 2.0A
#define INL_21	0.585		// 2.1A
#define INL_22	0.585		// 2.2A
#define INL_23	0.585		// 2.3A
#define INL_24	0.585		// 2.4A
#define INL_25	0.585		// 2.5A
#define INL_26	0.585		// 2.6A
#define INL_27	0.585		// 2.7A
#define INL_28	0.585		// 2.8A
#define INL_29	0.585		// 2.9A
#define INL_30	0.585		// 3.0A
#define INL_31	0.585		// 3.1A

#endif	// _PWS_E6114_H

