
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		userif.c
*	Project:	MCRP09_RX200 (ss) (YRotateItRX200_1s_V1)
*	Version:	1.0
*	Date:		19-11-2013 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	User interface
*******************************************************************************/

#define	_USERIF_C			// insert define BEFORE including header files

#include "globdef.h"
#include "userif.h"
#include "ges_eqp.h"

/* timing defines */
#define	LED_TIME_FUN	20	// 200ms led blinking time
#define	LED_TIME_ALL	3	// 30ms led blinking time

/* hardware related defines */
#define ERI1_NUM		VECT_SCI1_ERI1
#define ERI1_FLG		ICU.IR[ERI1_NUM].BIT.IR
#define	RX1_ERR			(ERI1_FLG != 0)
#define	RES_RX1_ERR		//ERI1_FLG = 0;
#define RXI1_NUM		VECT_SCI1_RXI1
#define RXI1_FLG		ICU.IR[RXI1_NUM].BIT.IR
#define	RX1_FULL		(RXI1_FLG != 0)
#define	RES_RX1_FULL	RXI1_FLG = 0;
#define TXI1_NUM		VECT_SCI1_TXI1
#define TXI1_FLG		ICU.IR[TXI1_NUM].BIT.IR
#define	TX1_EMPTY		(TXI1_FLG != 0)
#define	RES_TX1_EMPTY	TXI1_FLG = 0;
#define	RX1_OVR_ERR		(SCI1.SSR.BIT.ORER != 0)
#define	RES_RX1_OVR_ERR	{if(RX1_OVR_ERR) SCI1.SSR.BIT.ORER = 0;}
#define	RX1_FRA_ERR		(SCI1.SSR.BIT.FER != 0)
#define	RES_RX1_FRA_ERR	{if(RX1_FRA_ERR) SCI1.SSR.BIT.FER = 0;}
#define	RX1_PAR_ERR		(SCI1.SSR.BIT.PER != 0)
#define	RES_RX1_PAR_ERR	{if(RX1_PAR_ERR) SCI1.SSR.BIT.PER = 0;}
#define ERR_FLG_MSK		0x38	// B5, B4, B3
#define RX1_HW_ERR		((SCI1.SSR.BYTE & ERR_FLG_MSK) != 0)
#define	RES_RX1_HW_ERR	SCI1.SSR.BYTE &= ~ERR_FLG_MSK;	// {RES_RX1_OVR_ERR RES_RX1_FRA_ERR RES_RX1_PAR_ERR}
#define TX1_END			(SCI1.SSR.BIT.TEND != 0)
#define	SET_TX1			SCI1.SCR.BYTE |= 0xA0;	// {SCI1.SCR.BIT.TE = 1; SCI1.SCR.BIT.TIE = 1;}
#define	RES_TX1			SCI1.SCR.BYTE &= 0x5F;	// {SCI1.SCR.BIT.TE = 0; SCI1.SCR.BIT.TIE = 0;}
#define	SET_RX1			SCI1.SCR.BYTE |= 0x50;	// {SCI1.SCR.BIT.RE = 1; SCI1.SCR.BIT.RIE = 1;}
#define	RES_RX1			SCI1.SCR.BYTE &= 0xAF;	// {SCI1.SCR.BIT.RE = 0; SCI1.SCR.BIT.RIE = 0;}
#define	RX1_REG			SCI1.RDR
#define	TX1_REG			SCI1.TDR

/* serial communication protocol */
#define	N_CENT			0	/* slave identification number */
#define SEL_MAS			'?'	/* master frame identification character (question) */
#define SEL_SLA			'!'	/* slave frame identification character (answer OK) */
#define SEL_NOK			'#'	/* slave frame identification character (answer NOK) */
#define CCO				'c'	/* Check Com operation code */
#define NCO				'C'	/* Check Com operation code (answer) */
#define RBO				'b'	/* Read Byte operation code */
#define RWO				'w'	/* Read Word operation code */
#define WWO				'W'	/* Write Word operation code */
#define RW3				'y'	/* Read Word 3 operation code (parameter minimum values) */
#define RW4				'z'	/* Read Word 4 operation code (parameter maximum values) */
#define	LEN_MIN			5	/* shortest frame length, crc included */
#define	LEN_DEL			2	/* additional characters, data excluded, crc included */
#define	IND_LEN			0	/* string length index */
#define	IND_SEL			1	/* master/slave identificator index */
#define	IND_ADS			2	/* station address index */
#define	IND_COD			3	/* operation code index */
#define IND_ADD			4	/* data address index */
#define	IND_ND			5	/* data number index */
#define	IND_ID			6	/* data start index */

/* software related flags */
#define	RX1_ON			(fser1.bit.b0 != 0)
#define	RX1_OFF			(fser1.bit.b0 == 0)
#define	RX1_INI			(fser1.bit.b1 != 0)
#define	TX1_ON			(fser1.bit.b2 != 0)
#define	TX1_OFF			(fser1.bit.b2 == 0)
#define	RTX1_ON			(fser1.bit.b3 != 0)
#define	RTX1_OFF		(fser1.bit.b3 == 0)
#define GO_TX1			(fser1.bit.b4 != 0)
#define	SET_RX1_ON		fser1.bit.b0 = 1;
#define	SET_RX1_INI		fser1.bit.b1 = 1;
#define	SET_TX1_ON		fser1.bit.b2 = 1;
#define	SET_RTX1_ON		fser1.bit.b3 = 1;
#define SET_GO_TX1		fser1.bit.b4 = 1;
#define	RES_RX1_ON		fser1.bit.b0 = 0;
#define	RES_RX1_INI		fser1.bit.b1 = 0;
#define	RES_TX1_ON		fser1.bit.b2 = 0;
#define	RES_RTX1_ON		fser1.bit.b3 = 0;
#define RES_GO_TX1		fser1.bit.b4 = 0;

#define	START_TX1		{	SET_TX1					/* enable TX */				\
							rtx1_cnt = 0;			/* counter init */			\
							SET_TX1_ON				/* TX flag setting */		}
/*							TX1_REG = rtx1_buf[1];	 shift reg. loading */
#define	START_RX1		{	RES_RX1												\
							rtx1_cnt = 0;			/* counter init */			\
							RES_RX1_FULL										\
							RES_RX1_ERR											\
							RES_RX1_HW_ERR										\
							RES_RX1_INI				/* reset RX started flag */	\
							SET_RX1					/* enable RX */				\
							SET_RX1_ON				/* RX flag setting */		}

#define	RTX1_LEN		220	/* rx/tx buffer dim. */
#define	RX1_TIM			20	/* end of receive delay (1=0.1ms) */

const uint8_t
	crc_tab[256] = {		// DOW CRC table
		  0,  94, 188, 226,  97,  63, 221, 131, 194, 156, 126,  32, 163, 253,  31,  65,
		157, 195,  33, 127, 252, 162,  64,  30,  95,   1, 227, 189,  62,  96, 130, 220,
		 35, 125, 159, 193,  66,  28, 254, 160, 225, 191,  93,   3, 128, 222,  60,  98,
		190, 224,   2,  92, 223, 129,  99,  61, 124,  34, 192, 158,  29,  67, 161, 255,
		 70,  24, 250, 164,  39, 121, 155, 197, 132, 218,  56, 102, 229, 187,  89,   7,
		219, 133, 103,  57, 186, 228,   6,  88,  25,  71, 165, 251, 120,  38, 196, 154,
		101,  59, 217, 135,   4,  90, 184, 230, 167, 249,  27,  69, 198, 152, 122,  36,
		248, 166,  68,  26, 153, 199,  37, 123,  58, 100, 134, 216,  91,   5, 231, 185,
		140, 210,  48, 110, 237, 179,  81,  15,  78,  16, 242, 172,  47, 113, 147, 205,
		 17,  79, 173, 243, 112,  46, 204, 146, 211, 141, 111,  49, 178, 236,  14,  80,
		175, 241,  19,  77, 206, 144, 114,  44, 109,  51, 209, 143,  12,  82, 176, 238,
		 50, 108, 142, 208,  83,  13, 239, 177, 240, 174,  76,  18, 145, 207,  45, 115,
		202, 148, 118,  40, 171, 245,  23,  73,   8,  86, 180, 234, 105,  55, 213, 139,
		 87,   9, 235, 181,  54, 104, 138, 212, 149, 203,  41, 119, 244, 170,  72,  22,
		233, 183,  85,  11, 136, 214,  52, 106,  43, 117, 151, 201,  74,  20, 246, 168,
		116,  42, 200, 150,  21,  75, 169, 247, 182, 232,  10,  84, 215, 137, 107,  53};

byte_t
	fser1;

uint16_t
	rtx1_tim = 0,
	rtx1_cnt = 0,
	rtx1_num = 0;
uint8_t
	rtx1_buf[RTX1_LEN];				// rx/tx buffer (half duplex)
uint16_t
	add_par,
	dta_par;

/******************************************************************************
* Function Name: b2w
* Description  : 2 bytes vector conversion in a word
* Arguments    : two bytes vector *v
* Return Value : the short obtained by the union of the two bytes
******************************************************************************/
static uint16_t b2w(uint8_t *v)
{
	word_t	w;

	w.byte.h = v[0];
	w.byte.l = v[1];
	return(w.word);
}

/******************************************************************************
* Function Name: w2b
* Description  : 1 word conversion in a 2 bytes vector
* Arguments    : the word to convert and the address of the result vector
* Return Value : the address of the result vector
* Note         : it modifies the vector content
******************************************************************************/
static uint8_t *w2b(uint16_t w, uint8_t *v)
{
	word_t	x;

	x.word = w;
	v[0] = x.byte.h;
	v[1] = x.byte.l;
	return(v);
}

/******************************************************************************
* Function Name: cal_crc
* Description  : DOW CRC calculation
* Arguments    : the number of bytes to be taken into account and the vector
*                containing these bytes
* Return Value : CRC
* Note         : it uses the DOW CRC calculation table defined before
******************************************************************************/
static uint8_t cal_crc(uint8_t n, uint8_t *v)
{
	uint8_t	cks = 0, i;

	for(i = 0; i < n; i++)
	{
		cks = crc_tab[cks ^ v[i]];
	}
	return(cks);
}

/******************************************************************************
* Function Name: ini_232_pc
* Description  : serial communication init
* Arguments    : none
* Return Value : none
******************************************************************************/
void ini_232_pc(void)
{

	WAKEUP_SCI1

	for(rtx1_cnt = 0; rtx1_cnt < RTX1_LEN; rtx1_cnt++)
		rtx1_buf[rtx1_cnt] = 0;
	rtx1_cnt = 0;
	rtx1_tim = 0,
	rtx1_num = 0;

	// reset some flags
	SCI1.SCR.BYTE=0x00;			// disable any tx/rx, set internal clock

	// ports setup
	PORT1.PMR.BIT.B6 = 0;		// PIN14 P16 TXD1 OUT TX
	PORT1.PMR.BIT.B5 = 0;		// PIN15 P15 RXD1 IN  RX
	PORT1.PDR.BIT.B6 = 1;		// PIN14 P16 TXD1 OUT TX
	PORT1.PDR.BIT.B5 = 0;		// PIN15 P15 RXD1 IN  RX

	// pin assignment to the peripheral
	MPC.PWPR.BYTE = 0x00;		// enable writing BIT6
	MPC.PWPR.BYTE = 0x40;		// enable writing MPC registers
	MPC.P16PFS.BYTE = 0x0A;		// PIN14 P16 TXD1 OUT TX
	MPC.P15PFS.BYTE = 0x0A;		// PIN15 P15 RXD1 IN  RX
	MPC.PWPR.BYTE = 0x80;		// disable writing MPC registers

	// ports setup
	PORT1.PMR.BIT.B6 = 1;		// PIN14 P16 TXD1 OUT TX
	PORT1.PMR.BIT.B5 = 1;		// PIN15 P15 RXD1 IN  RX

	// set serial mode and baud rate
	SCI1.SCR.BYTE = 0x00;		// disable any tx/rx, set internal clock
	SCI1.SMR.BYTE=0x00;			// Asynchronous, N81, clock not divided
	//SCI1.SMR.BYTE = 0x01;		// Asynchronous, N81, clock divided by 4
	SCI1.SEMR.BYTE = 0x00;		// base clock=16, no noise canc
	SCI1.BRR=103;					// 9600baud @32MHz
	//SCI1.BRR = 25;				// 9600baud @8MHz

	DELAY1MS(1);				// wait at least 1 bit time

	//RES_GO_TX1
	START_RX1					// start rx

}

/******************************************************************************
* Function Name: poll_232_pc
* Description  : serial communication polling
* Arguments    : none
* Return Value : none
* Note         : to be called in the control interrupt
******************************************************************************/
void poll_232_pc(void)
{

	if(TX1_ON)
	{
		if(TX1_EMPTY)							//	if buffer empty ...
		{
			RES_TX1_EMPTY						//	... reset empty buffer flag ...
			if(rtx1_cnt < rtx1_num)				//	... if data not finished ...
			{
				TX1_REG = rtx1_buf[rtx1_cnt++];	//	... loading buffer
			}
		}
		if(TX1_END)								//	if tx end ...
		{
			RES_TX1								//	... tx disable ...
			RES_TX1_ON							//	... end tx ...
			START_RX1							//	... rx enable
		}
	}
	else if(RX1_ON)
	{
		if(RX1_ERR)								//	if errors ...
		{
			START_RX1							//	... re-start
		}
		else if(RX1_FULL)						//	if data received ...
		{
			RES_RX1_FULL						//	... reset flag ...
			SET_RX1_INI							//	... rx begun flag setting ...
			rtx1_tim = RX1_TIM;					//	... delay timer reset ...
			if(rtx1_cnt < RTX1_LEN)				//	... if enough space ...
			{
				rtx1_buf[rtx1_cnt++] = RX1_REG;	//		... received data saving ...
			}
			else								//	... if no space ...
			{
				RES_RX1							//		... rx disable ...
				RES_RX1_ON						//		... rx end
			}
		}
		else if(RX1_INI)						//	if rx begun ...
		{
			if(rtx1_tim > 0)					//	... if still time ...
			{
				--rtx1_tim;						//		... wait
			}
			else								//	... if delay elapsed ...
			{
				RES_RX1							//		... rx disable ...
				RES_RX1_ON						//		... end rx
			}
		}
	}

}

/******************************************************************************
* Function Name: ges_232_pc
* Description  : serial communication management
* Arguments    : none
* Return Value : none
* Note         : to be called in the main loop;
*                the slave is always waiting for commands (reception)
*                when a commend is received, it is decoded and, if it make sense,
*                executed; if the command is correctly received or if the errors
*                are in the data or in the addresses, the slave send an answer.
******************************************************************************/
void ges_232_pc(void)
{
	static uint16_t	add, nda, i;

//	if(GO_TX1) {
//		RES_GO_TX1
//		START_TX1
//	}
//	else
	if(RX1_INI && RX1_OFF)			// end RX: decoding and start TX
	{
		RES_RX1_INI					// reset flag

		// no answer cases
		rtx1_num = rtx1_buf[IND_LEN];
		if(	(rtx1_num < LEN_MIN) 			||
			(rtx1_num > rtx1_cnt) 			||
			(rtx1_buf[IND_SEL] != SEL_MAS) 	||
			(rtx1_buf[IND_ADS] != N_CENT)	)
		{
			START_RX1
			return;
		}
		if(rtx1_buf[rtx1_num - 1] != cal_crc((rtx1_num - 1), rtx1_buf))
		{
			START_RX1
			return;
		}

		// operation codes select and answer coding
		SET_RTX1_ON							// flag communication on
		rtx1_buf[IND_SEL] = SEL_SLA;
	    rtx1_num = LEN_MIN;					// frame lenght
		switch(rtx1_buf[IND_COD])
		{
			case CCO:	// check
			case NCO:	// check
				rtx1_buf[IND_COD] = NCO;		// answer of programs without auto tuning
				break;
			case RWO:	// answer coding
				add = rtx1_buf[IND_ADD];
				nda = rtx1_buf[IND_ND];
				if(add < NUM_PAR)		// reading from parameters table
				{
					if((nda > NUM_PAR) || (add > (NUM_PAR - nda)))
					{
						rtx1_buf[IND_SEL] = SEL_NOK;
						break;
					}
					for(i = 0; i < nda; i++)
					{
						w2b(par_vec[add + i], &(rtx1_buf[IND_ID + (i << 1)]));
					}
				    rtx1_num += LEN_DEL + (nda << 1);// frame lenght
				}
				else					// reading from ram table
				{
					add -= NUM_PAR;
					if((nda > N_RAM_READ) || (add > (N_RAM_READ - nda)))
					{
						rtx1_buf[IND_SEL] = SEL_NOK;
						break;
					}
					for(i = 0; i < nda; i++)
					{
						w2b(UIF_R.ram_tab[add + i], &(rtx1_buf[IND_ID + (i << 1)]));
					}
				    rtx1_num += LEN_DEL + (nda << 1);// frame lenght
				}
				break;
			case WWO:	// answer coding
				add = rtx1_buf[IND_ADD];
				nda = rtx1_buf[IND_ND];
				if(add < NUM_PAR)		// writing in parameters table
				{
					if((nda > NUM_PAR) || (add > (NUM_PAR - nda)))
					{
						rtx1_buf[IND_SEL] = SEL_NOK;
						break;
					}
					for(i = 0; i < nda; i++)
					{
						dta_par = b2w(&(rtx1_buf[IND_ID + (i << 1)]));
						add_par = add + i;
						if(par_vec[add_par] != dta_par)
						{
							eqp_set_par(dta_par, add_par);
						}
					}
					dta_par = 0;
					add_par = 0;
				}
				else						// writing in ram table
				{
					add -= NUM_PAR;
					if((nda > N_RAM_WRITE) || (add > (N_RAM_WRITE - nda)))
					{
						rtx1_buf[IND_SEL] = SEL_NOK;
						break;
					}
					for(i = 0; i < nda; i++)
					{
						UIF_W.ram_tab[add + i] = b2w(&(rtx1_buf[IND_ID + (i << 1)]));
					}
				}
				break;
			case RW3:	// reading parameters minimum values
				add = rtx1_buf[IND_ADD];
				nda = rtx1_buf[IND_ND];
				if((add >= NUM_PAR) || (nda > (NUM_PAR - add)))
				{
					rtx1_buf[IND_SEL] = SEL_NOK;
					break;
				}
				for(i = 0; i < nda; i++)
				{
					w2b(par_min[add + i], &(rtx1_buf[IND_ID + (i << 1)]));
				}
			    rtx1_num += LEN_DEL;
			    rtx1_num += nda << 1;
				break;
			case RW4:	// reading parameters maximum values
				add = rtx1_buf[IND_ADD];
				nda = rtx1_buf[IND_ND];
				if((add >= NUM_PAR) || (nda > (NUM_PAR - add)))
				{
					rtx1_buf[IND_SEL] = SEL_NOK;
					break;
				}
				for(i = 0; i < nda; i++)
				{
					w2b(par_max[add + i], &(rtx1_buf[IND_ID + (i << 1)]));
				}
			    rtx1_num += LEN_DEL;
			    rtx1_num += nda << 1;
				break;
			default:
				rtx1_buf[IND_SEL] = SEL_NOK;
				break;
		}

		// crc calc. and answer transmission
		rtx1_buf[IND_LEN] = rtx1_num;
		rtx1_buf[rtx1_num - 1] = cal_crc(rtx1_num - 1, rtx1_buf);
		rtx1_buf[rtx1_num] = 0;			// ("C" string)
//		SET_GO_TX1
		START_TX1
		return;
    }

}

/******************************************************************************
* Function Name: ges_led
* Description  : led blinking management
* Arguments    : none
* Return Value : none
* Note         : to be called in the main loop
******************************************************************************/
void ges_led(void)
{
	static uint16_t led_cnt;
	uint16_t led_time;

	if(0 == UIF_R.var.all) 
	{
		led_time = LED_TIME_FUN;
	}
	else
	{
		led_time = LED_TIME_ALL;
	}
	if(led_cnt < led_time)
	{
		++led_cnt;
	}
	else
	{
		led_cnt = 0;
		if(DL3_OFF)
		{
			SET_DL3
		}
		else
		{
			CLR_DL3
		}
	}
}

