
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		userif.h
*	Project:	MCRP09_RX200 (ss) (YRotateItRX200_1s_V1)
*	Version:	1.0
*	Date:		19-11-2013 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	Header for userif.c
*******************************************************************************/

#ifndef _USERIF_H
#define _USERIF_H

#include "typedefine.h"

/******************************************************************************
Macro definitions
******************************************************************************/
#define N_RAM_READ		20
#define N_RAM_WRITE		8


/******************************************************************************
Typedef definitions
******************************************************************************/
typedef union
{
	uint16_t		ram_tab[N_RAM_READ];
	struct
	{
		int16_t		rif;		//  0	internal speed reference
		int16_t		rpm;		//  1	measured speed
		int16_t		fre;		//  2	imposed frequency
		int16_t		id;			//  3	direct current
		int16_t		iq;			//  4	quadrature current
		int16_t		vd;			//  5	direct voltage
		int16_t		vq;			//  6	quadrature voltage
		int16_t		vb;			//  7	bus voltage
		int16_t		all;		//  8	alarm
		int16_t		flg;		//  9	flags
		int16_t		toti;		// 10	current vector amplitude
		int16_t		totv;		// 11	voltage vector amplitude
		int16_t		du0;		// 12
		int16_t		du1;		// 13
		int16_t		du2;		// 14
		int16_t		du3;		// 15
		int16_t		du4;		// 16
		int16_t		du5;		// 17
		int16_t		du6;		// 18
		int16_t		du7;		// 19
	}				var;
}	UIF_R_t;

typedef union
{
	uint16_t		ram_tab[N_RAM_WRITE];
	struct
	{
		int16_t		trg;		//  0	trigger
		int16_t		mod;		//  1	mode
		int16_t		rif;		//  2	speed reference
		int16_t		cra;		//  3	current ratio
		int16_t		sel;		//  4	variable and time scale selection
		int16_t		du0;		//  5
		int16_t		du1;		//  6
		int16_t		du2;		//  7
	}				var;
}	UIF_W_t;

/******************************************************************************
Variables definitions/declarations
******************************************************************************/

#ifndef _USERIF_C
extern
#endif	// _USERIF_C
UIF_R_t		UIF_R;

#ifndef _USERIF_C
extern
#endif	// _USERIF_C
UIF_W_t		UIF_W;

/******************************************************************************
Functions Prototypes
******************************************************************************/

/******************************************************************************
* Function Name: ini_232_pc
* Description  : serial communication init
* Arguments    : none
* Return Value : none
******************************************************************************/
void ini_232_pc(void);

/******************************************************************************
* Function Name: poll_232_pc
* Description  : serial communication polling
* Arguments    : none
* Return Value : none
* Note         : to be called in the control interrupt
******************************************************************************/
void poll_232_pc(void);

/******************************************************************************
* Function Name: ges_232_pc
* Description  : serial communication management
* Arguments    : none
* Return Value : none
* Note         : to be called in the main loop
******************************************************************************/
void ges_232_pc(void);

/******************************************************************************
* Function Name: ges_led
* Description  : led blinking management
* Arguments    : none
* Return Value : none
* Note         : to be called in the main loop
******************************************************************************/
void ges_led(void);

#endif	// _USERIF_H

